@@ -33,6 +33,7 @@
 #include "config.h"
 #include "version.h"
 
+#include <err.h>
 #include <errno.h>
 #include <locale.h>
 #include <string.h>
@@ -101,10 +102,17 @@ static void serve( int host_fd,
 
 static int run_server( const char *desired_ip, const char *desired_port,
 		       const string &command_path, char *command_argv[],
-		       const int colors, bool verbose, bool with_motd );
+		       const int colors, unsigned int verbose, bool with_motd );
 
 using namespace std;
 
+static void print_version( FILE *file )
+{
+  fprintf( file, "mosh-server (%s) [build %s]\n", PACKAGE_STRING, BUILD_VERSION );
+  fprintf( file, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
+  fprintf( file, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n" );
+}
+
 static void print_usage( FILE *stream, const char *argv0 )
 {
   fprintf( stream, "Usage: %s new [-s] [-v] [-i LOCALADDR] [-p PORT[:PORT2]] [-c COLORS] [-l NAME=VALUE] [-- COMMAND...]\n", argv0 );
@@ -170,16 +178,20 @@ int main( int argc, char *argv[] )
   string command_path;
   char **command_argv = NULL;
   int colors = 0;
-  bool verbose = false; /* don't close stdin/stdout/stderr */
+  unsigned int verbose = 0; /* don't close stdin/stdout/stderr */
   /* Will cause mosh-server not to correctly detach on old versions of sshd. */
   list<string> locale_vars;
 
   /* strip off command */
-  for ( int i = 0; i < argc; i++ ) {
+  for ( int i = 1; i < argc; i++ ) {
     if ( 0 == strcmp( argv[ i ], "--help" ) || 0 == strcmp( argv[ i ], "-h" ) ) {
       print_usage( stdout, argv[ 0 ] );
       exit( 0 );
     }
+    if ( 0 == strcmp( argv[ i ], "--version" ) ) {
+      print_version( stdout );
+      exit( 0 );
+    }
     if ( 0 == strcmp( argv[ i ], "--" ) ) { /* -- is mandatory */
       if ( i != argc - 1 ) {
 	command_argv = argv + i + 1;
@@ -230,7 +242,7 @@ int main( int argc, char *argv[] )
 	}
 	break;
       case 'v':
-	verbose = true;
+	verbose++;
 	break;
       case 'l':
 	locale_vars.push_back( string( optarg ) );
@@ -270,7 +282,7 @@ int main( int argc, char *argv[] )
     /* get shell name */
     const char *shell = getenv( "SHELL" );
     if ( shell == NULL ) {
-      struct passwd *pw = getpwuid( geteuid() );
+      struct passwd *pw = getpwuid( getuid() );
       if ( pw == NULL ) {
 	perror( "getpwuid" );
 	exit( 1 );
@@ -354,7 +366,7 @@ int main( int argc, char *argv[] )
 
 static int run_server( const char *desired_ip, const char *desired_port,
 		       const string &command_path, char *command_argv[],
-		       const int colors, bool verbose, bool with_motd ) {
+		       const int colors, unsigned int verbose, bool with_motd ) {
   /* get network idle timeout */
   long network_timeout = 0;
   char *timeout_envar = getenv( "MOSH_SERVER_NETWORK_TMOUT" );
@@ -388,8 +400,6 @@ static int run_server( const char *desired_ip, const char *desired_port,
   if ( ioctl( STDIN_FILENO, TIOCGWINSZ, &window_size ) < 0 ||
        window_size.ws_col == 0 ||
        window_size.ws_row == 0 ) {
-    fprintf( stderr, "Server started without pseudo-terminal. Opening 80x24 terminal.\n" );
-
     /* Fill in sensible defaults. */
     /* They will be overwritten by client on first connection. */
     memset( &window_size, 0, sizeof( window_size ) );
@@ -404,12 +414,18 @@ static int run_server( const char *desired_ip, const char *desired_port,
   Network::UserStream blank;
   ServerConnection *network = new ServerConnection( terminal, blank, desired_ip, desired_port );
 
-  if ( verbose ) {
-    network->set_verbose();
-  }
+  network->set_verbose( verbose );
+  Select::set_verbose( verbose );
 
-  printf( "\nMOSH CONNECT %s %s\n", network->port().c_str(), network->get_key().c_str() );
-  fflush( stdout );
+  /*
+   * If mosh-server is run on a pty, then typeahead may echo and break mosh.pl's
+   * detection of the MOSH CONNECT message.  Print it on a new line to bodge
+   * around that.
+   */
+  if ( isatty( STDIN_FILENO ) ) {
+    puts( "\r\n" );
+  }
+  printf( "MOSH CONNECT %s %s\n", network->port().c_str(), network->get_key().c_str() );
 
   /* don't let signals kill us */
   struct sigaction sa;
@@ -421,27 +437,36 @@ static int run_server( const char *desired_ip, const char *desired_port,
 
 
   /* detach from terminal */
+  fflush( stdout );
+  fflush( stderr );
   pid_t the_pid = fork();
   if ( the_pid < 0 ) {
     perror( "fork" );
   } else if ( the_pid > 0 ) {
+    fprintf( stderr, "\nmosh-server (%s) [build %s]\n", PACKAGE_STRING, BUILD_VERSION );
+    fprintf( stderr, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
+    fprintf( stderr, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n\n" );
+
+    fprintf( stderr, "[mosh-server detached, pid = %d]\n", static_cast<int>(the_pid) );
+#ifndef HAVE_IUTF8
+    fprintf( stderr, "\nWarning: termios IUTF8 flag not defined.\nCharacter-erase of multibyte character sequence\nprobably does not work properly on this platform.\n" );
+#endif /* HAVE_IUTF8 */
+
+    fflush( stdout );
+    fflush( stderr );
+    if ( isatty( STDOUT_FILENO ) ) {
+      tcdrain( STDOUT_FILENO );
+    }
+    if ( isatty( STDERR_FILENO ) ) {
+      tcdrain( STDERR_FILENO );
+    }
     _exit( 0 );
   }
 
-  fprintf( stderr, "\nmosh-server (%s) [build %s]\n", PACKAGE_STRING, BUILD_VERSION );
-  fprintf( stderr, "Copyright 2012 Keith Winstein <mosh-devel@mit.edu>\n" );
-  fprintf( stderr, "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\nThis is free software: you are free to change and redistribute it.\nThere is NO WARRANTY, to the extent permitted by law.\n\n" );
-
-  fprintf( stderr, "[mosh-server detached, pid = %d]\n", (int)getpid() );
-
   int master;
 
-#ifndef HAVE_IUTF8
-  fprintf( stderr, "\nWarning: termios IUTF8 flag not defined.\nCharacter-erase of multibyte character sequence\nprobably does not work properly on this platform.\n" );
-#endif /* HAVE_IUTF8 */
-
   /* close file descriptors */
-  if ( !verbose ) {
+  if ( verbose == 0 ) {
     /* Necessary to properly detach on old versions of sshd (e.g. RHEL/CentOS 5.0). */
     int nullfd;
 
@@ -465,7 +490,7 @@ static int run_server( const char *desired_ip, const char *desired_port,
   }
 
   char utmp_entry[ 64 ] = { 0 };
-  snprintf( utmp_entry, 64, "mosh [%d]", getpid() );
+  snprintf( utmp_entry, 64, "mosh [%ld]", static_cast<long int>( getpid() ) );
 
   /* Fork child process */
   pid_t child = forkpty( &master, NULL, NULL, &window_size );
@@ -549,6 +574,15 @@ static int run_server( const char *desired_ip, const char *desired_port,
   } else {
     /* parent */
 
+    /* Drop unnecessary privileges */
+#ifdef HAVE_PLEDGE
+    /* OpenBSD pledge() syscall */
+    if ( pledge( "stdio inet tty", NULL )) {
+      perror( "pledge() failed" );
+      exit( 1 );
+    }
+#endif
+
 #ifdef HAVE_UTEMPTER
     /* make utmp entry */
     utempter_add_record( master, utmp_entry );
@@ -676,13 +710,13 @@ static void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &
 	      struct winsize window_size;
 	      if ( ioctl( host_fd, TIOCGWINSZ, &window_size ) < 0 ) {
 		perror( "ioctl TIOCGWINSZ" );
-		return;
+		network.start_shutdown();
 	      }
 	      window_size.ws_col = res->width;
 	      window_size.ws_row = res->height;
 	      if ( ioctl( host_fd, TIOCSWINSZ, &window_size ) < 0 ) {
 		perror( "ioctl TIOCSWINSZ" );
-		return;
+		network.start_shutdown();
 	      }
 	    }
 	    terminal_to_host += terminal.act( action );
@@ -758,7 +792,7 @@ static void serve( int host_fd, Terminal::Complete &terminal, ServerConnection &
 
       /* write user input and terminal writeback to the host */
       if ( swrite( host_fd, terminal_to_host.c_str(), terminal_to_host.length() ) < 0 ) {
-	break;
+	network.start_shutdown();
       }
 
       bool idle_shutdown = false;
@@ -872,7 +906,7 @@ static void chdir_homedir( void )
 {
   const char *home = getenv( "HOME" );
   if ( home == NULL ) {
-    struct passwd *pw = getpwuid( geteuid() );
+    struct passwd *pw = getpwuid( getuid() );
     if ( pw == NULL ) {
       perror( "getpwuid" );
       return; /* non-fatal */
@@ -907,7 +941,7 @@ static void warn_unattached( const string & ignore_entry )
 {
 #ifdef HAVE_UTMPX_H
   /* get username */
-  const struct passwd *pw = getpwuid( geteuid() );
+  const struct passwd *pw = getpwuid( getuid() );
   if ( pw == NULL ) {
     perror( "getpwuid" );
     /* non-fatal */
